// Copyright 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

library system_chrome_mojom;

import 'dart:async';

import 'package:mojo/bindings.dart' as bindings;
import 'package:mojo/core.dart' as core;

class DeviceOrientation extends bindings.MojoEnum {
  static const DeviceOrientation portraitUp = const DeviceOrientation._(1);
  static const DeviceOrientation landscapeLeft = const DeviceOrientation._(2);
  static const DeviceOrientation portraitDown = const DeviceOrientation._(4);
  static const DeviceOrientation landscapeRight = const DeviceOrientation._(8);

  const DeviceOrientation._(int v) : super(v);

  static const Map<String, DeviceOrientation> valuesMap = const {
    "portraitUp": portraitUp,
    "landscapeLeft": landscapeLeft,
    "portraitDown": portraitDown,
    "landscapeRight": landscapeRight,
  };
  static const List<DeviceOrientation> values = const [
    portraitUp,
    landscapeLeft,
    portraitDown,
    landscapeRight,
  ];

  static DeviceOrientation valueOf(String name) => valuesMap[name];

  factory DeviceOrientation(int v) {
    switch (v) {
      case 1:
        return portraitUp;
      case 2:
        return landscapeLeft;
      case 4:
        return portraitDown;
      case 8:
        return landscapeRight;
      default:
        return null;
    }
  }

  static DeviceOrientation decode(bindings.Decoder decoder0, int offset) {
    int v = decoder0.decodeUint32(offset);
    DeviceOrientation result = new DeviceOrientation(v);
    if (result == null) {
      throw new bindings.MojoCodecError(
          'Bad value $v for enum DeviceOrientation.');
    }
    return result;
  }

  String toString() {
    switch(this) {
      case portraitUp:
        return 'DeviceOrientation.portraitUp';
      case landscapeLeft:
        return 'DeviceOrientation.landscapeLeft';
      case portraitDown:
        return 'DeviceOrientation.portraitDown';
      case landscapeRight:
        return 'DeviceOrientation.landscapeRight';
    }
  }

  int toJson() => mojoEnumValue;
}

class SystemUiOverlay extends bindings.MojoEnum {
  static const SystemUiOverlay top = const SystemUiOverlay._(1);
  static const SystemUiOverlay bottom = const SystemUiOverlay._(2);

  const SystemUiOverlay._(int v) : super(v);

  static const Map<String, SystemUiOverlay> valuesMap = const {
    "top": top,
    "bottom": bottom,
  };
  static const List<SystemUiOverlay> values = const [
    top,
    bottom,
  ];

  static SystemUiOverlay valueOf(String name) => valuesMap[name];

  factory SystemUiOverlay(int v) {
    switch (v) {
      case 1:
        return top;
      case 2:
        return bottom;
      default:
        return null;
    }
  }

  static SystemUiOverlay decode(bindings.Decoder decoder0, int offset) {
    int v = decoder0.decodeUint32(offset);
    SystemUiOverlay result = new SystemUiOverlay(v);
    if (result == null) {
      throw new bindings.MojoCodecError(
          'Bad value $v for enum SystemUiOverlay.');
    }
    return result;
  }

  String toString() {
    switch(this) {
      case top:
        return 'SystemUiOverlay.top';
      case bottom:
        return 'SystemUiOverlay.bottom';
    }
  }

  int toJson() => mojoEnumValue;
}



class _SystemChromeSetPreferredOrientationsParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int deviceOrientationMask = 0;

  _SystemChromeSetPreferredOrientationsParams() : super(kVersions.last.size);

  static _SystemChromeSetPreferredOrientationsParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SystemChromeSetPreferredOrientationsParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SystemChromeSetPreferredOrientationsParams result = new _SystemChromeSetPreferredOrientationsParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.deviceOrientationMask = decoder0.decodeUint32(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeUint32(deviceOrientationMask, 8);
  }

  String toString() {
    return "_SystemChromeSetPreferredOrientationsParams("
           "deviceOrientationMask: $deviceOrientationMask" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["deviceOrientationMask"] = deviceOrientationMask;
    return map;
  }
}


class SystemChromeSetPreferredOrientationsResponseParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  bool success = false;

  SystemChromeSetPreferredOrientationsResponseParams() : super(kVersions.last.size);

  static SystemChromeSetPreferredOrientationsResponseParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static SystemChromeSetPreferredOrientationsResponseParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    SystemChromeSetPreferredOrientationsResponseParams result = new SystemChromeSetPreferredOrientationsResponseParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.success = decoder0.decodeBool(8, 0);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeBool(success, 8, 0);
  }

  String toString() {
    return "SystemChromeSetPreferredOrientationsResponseParams("
           "success: $success" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["success"] = success;
    return map;
  }
}


class _SystemChromeSetEnabledSystemUiOverlaysParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int overlays = 0;

  _SystemChromeSetEnabledSystemUiOverlaysParams() : super(kVersions.last.size);

  static _SystemChromeSetEnabledSystemUiOverlaysParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SystemChromeSetEnabledSystemUiOverlaysParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SystemChromeSetEnabledSystemUiOverlaysParams result = new _SystemChromeSetEnabledSystemUiOverlaysParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.overlays = decoder0.decodeUint32(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeUint32(overlays, 8);
  }

  String toString() {
    return "_SystemChromeSetEnabledSystemUiOverlaysParams("
           "overlays: $overlays" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["overlays"] = overlays;
    return map;
  }
}


class SystemChromeSetEnabledSystemUiOverlaysResponseParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  bool success = false;

  SystemChromeSetEnabledSystemUiOverlaysResponseParams() : super(kVersions.last.size);

  static SystemChromeSetEnabledSystemUiOverlaysResponseParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static SystemChromeSetEnabledSystemUiOverlaysResponseParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    SystemChromeSetEnabledSystemUiOverlaysResponseParams result = new SystemChromeSetEnabledSystemUiOverlaysResponseParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.success = decoder0.decodeBool(8, 0);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeBool(success, 8, 0);
  }

  String toString() {
    return "SystemChromeSetEnabledSystemUiOverlaysResponseParams("
           "success: $success" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["success"] = success;
    return map;
  }
}

const int _SystemChrome_setPreferredOrientationsName = 0;
const int _SystemChrome_setEnabledSystemUiOverlaysName = 1;

abstract class SystemChrome {
  static const String serviceName = "flutter::platform::SystemChrome";
  dynamic setPreferredOrientations(int deviceOrientationMask,[Function responseFactory = null]);
  dynamic setEnabledSystemUiOverlays(int overlays,[Function responseFactory = null]);
}


class _SystemChromeProxyImpl extends bindings.Proxy {
  _SystemChromeProxyImpl.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) : super.fromEndpoint(endpoint);

  _SystemChromeProxyImpl.fromHandle(core.MojoHandle handle) :
      super.fromHandle(handle);

  _SystemChromeProxyImpl.unbound() : super.unbound();

  static _SystemChromeProxyImpl newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For _SystemChromeProxyImpl"));
    return new _SystemChromeProxyImpl.fromEndpoint(endpoint);
  }

  void handleResponse(bindings.ServiceMessage message) {
    switch (message.header.type) {
      case _SystemChrome_setPreferredOrientationsName:
        var r = SystemChromeSetPreferredOrientationsResponseParams.deserialize(
            message.payload);
        if (!message.header.hasRequestId) {
          proxyError("Expected a message with a valid request Id.");
          return;
        }
        Completer c = completerMap[message.header.requestId];
        if (c == null) {
          proxyError(
              "Message had unknown request Id: ${message.header.requestId}");
          return;
        }
        completerMap.remove(message.header.requestId);
        if (c.isCompleted) {
          proxyError("Response completer already completed");
          return;
        }
        c.complete(r);
        break;
      case _SystemChrome_setEnabledSystemUiOverlaysName:
        var r = SystemChromeSetEnabledSystemUiOverlaysResponseParams.deserialize(
            message.payload);
        if (!message.header.hasRequestId) {
          proxyError("Expected a message with a valid request Id.");
          return;
        }
        Completer c = completerMap[message.header.requestId];
        if (c == null) {
          proxyError(
              "Message had unknown request Id: ${message.header.requestId}");
          return;
        }
        completerMap.remove(message.header.requestId);
        if (c.isCompleted) {
          proxyError("Response completer already completed");
          return;
        }
        c.complete(r);
        break;
      default:
        proxyError("Unexpected message type: ${message.header.type}");
        close(immediate: true);
        break;
    }
  }

  String toString() {
    var superString = super.toString();
    return "_SystemChromeProxyImpl($superString)";
  }
}


class _SystemChromeProxyCalls implements SystemChrome {
  _SystemChromeProxyImpl _proxyImpl;

  _SystemChromeProxyCalls(this._proxyImpl);
    dynamic setPreferredOrientations(int deviceOrientationMask,[Function responseFactory = null]) {
      var params = new _SystemChromeSetPreferredOrientationsParams();
      params.deviceOrientationMask = deviceOrientationMask;
      return _proxyImpl.sendMessageWithRequestId(
          params,
          _SystemChrome_setPreferredOrientationsName,
          -1,
          bindings.MessageHeader.kMessageExpectsResponse);
    }
    dynamic setEnabledSystemUiOverlays(int overlays,[Function responseFactory = null]) {
      var params = new _SystemChromeSetEnabledSystemUiOverlaysParams();
      params.overlays = overlays;
      return _proxyImpl.sendMessageWithRequestId(
          params,
          _SystemChrome_setEnabledSystemUiOverlaysName,
          -1,
          bindings.MessageHeader.kMessageExpectsResponse);
    }
}


class SystemChromeProxy implements bindings.ProxyBase {
  final bindings.Proxy impl;
  SystemChrome ptr;

  SystemChromeProxy(_SystemChromeProxyImpl proxyImpl) :
      impl = proxyImpl,
      ptr = new _SystemChromeProxyCalls(proxyImpl);

  SystemChromeProxy.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) :
      impl = new _SystemChromeProxyImpl.fromEndpoint(endpoint) {
    ptr = new _SystemChromeProxyCalls(impl);
  }

  SystemChromeProxy.fromHandle(core.MojoHandle handle) :
      impl = new _SystemChromeProxyImpl.fromHandle(handle) {
    ptr = new _SystemChromeProxyCalls(impl);
  }

  SystemChromeProxy.unbound() :
      impl = new _SystemChromeProxyImpl.unbound() {
    ptr = new _SystemChromeProxyCalls(impl);
  }

  factory SystemChromeProxy.connectToService(
      bindings.ServiceConnector s, String url, [String serviceName]) {
    SystemChromeProxy p = new SystemChromeProxy.unbound();
    s.connectToService(url, p, serviceName);
    return p;
  }

  static SystemChromeProxy newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For SystemChromeProxy"));
    return new SystemChromeProxy.fromEndpoint(endpoint);
  }

  String get serviceName => SystemChrome.serviceName;

  Future close({bool immediate: false}) => impl.close(immediate: immediate);

  Future responseOrError(Future f) => impl.responseOrError(f);

  Future get errorFuture => impl.errorFuture;

  int get version => impl.version;

  Future<int> queryVersion() => impl.queryVersion();

  void requireVersion(int requiredVersion) {
    impl.requireVersion(requiredVersion);
  }

  String toString() {
    return "SystemChromeProxy($impl)";
  }
}


class SystemChromeStub extends bindings.Stub {
  SystemChrome _impl = null;

  SystemChromeStub.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint, [this._impl])
      : super.fromEndpoint(endpoint);

  SystemChromeStub.fromHandle(core.MojoHandle handle, [this._impl])
      : super.fromHandle(handle);

  SystemChromeStub.unbound() : super.unbound();

  static SystemChromeStub newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For SystemChromeStub"));
    return new SystemChromeStub.fromEndpoint(endpoint);
  }


  SystemChromeSetPreferredOrientationsResponseParams _SystemChromeSetPreferredOrientationsResponseParamsFactory(bool success) {
    var mojo_factory_result = new SystemChromeSetPreferredOrientationsResponseParams();
    mojo_factory_result.success = success;
    return mojo_factory_result;
  }
  SystemChromeSetEnabledSystemUiOverlaysResponseParams _SystemChromeSetEnabledSystemUiOverlaysResponseParamsFactory(bool success) {
    var mojo_factory_result = new SystemChromeSetEnabledSystemUiOverlaysResponseParams();
    mojo_factory_result.success = success;
    return mojo_factory_result;
  }

  dynamic handleMessage(bindings.ServiceMessage message) {
    if (bindings.ControlMessageHandler.isControlMessage(message)) {
      return bindings.ControlMessageHandler.handleMessage(this,
                                                          0,
                                                          message);
    }
    assert(_impl != null);
    switch (message.header.type) {
      case _SystemChrome_setPreferredOrientationsName:
        var params = _SystemChromeSetPreferredOrientationsParams.deserialize(
            message.payload);
        var response = _impl.setPreferredOrientations(params.deviceOrientationMask,_SystemChromeSetPreferredOrientationsResponseParamsFactory);
        if (response is Future) {
          return response.then((response) {
            if (response != null) {
              return buildResponseWithId(
                  response,
                  _SystemChrome_setPreferredOrientationsName,
                  message.header.requestId,
                  bindings.MessageHeader.kMessageIsResponse);
            }
          });
        } else if (response != null) {
          return buildResponseWithId(
              response,
              _SystemChrome_setPreferredOrientationsName,
              message.header.requestId,
              bindings.MessageHeader.kMessageIsResponse);
        }
        break;
      case _SystemChrome_setEnabledSystemUiOverlaysName:
        var params = _SystemChromeSetEnabledSystemUiOverlaysParams.deserialize(
            message.payload);
        var response = _impl.setEnabledSystemUiOverlays(params.overlays,_SystemChromeSetEnabledSystemUiOverlaysResponseParamsFactory);
        if (response is Future) {
          return response.then((response) {
            if (response != null) {
              return buildResponseWithId(
                  response,
                  _SystemChrome_setEnabledSystemUiOverlaysName,
                  message.header.requestId,
                  bindings.MessageHeader.kMessageIsResponse);
            }
          });
        } else if (response != null) {
          return buildResponseWithId(
              response,
              _SystemChrome_setEnabledSystemUiOverlaysName,
              message.header.requestId,
              bindings.MessageHeader.kMessageIsResponse);
        }
        break;
      default:
        throw new bindings.MojoCodecError("Unexpected message name");
        break;
    }
    return null;
  }

  SystemChrome get impl => _impl;
  set impl(SystemChrome d) {
    assert(_impl == null);
    _impl = d;
  }

  String toString() {
    var superString = super.toString();
    return "SystemChromeStub($superString)";
  }

  int get version => 0;
}


