// Copyright 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

library system_sound_mojom;

import 'dart:async';

import 'package:mojo/bindings.dart' as bindings;
import 'package:mojo/core.dart' as core;

class SystemSoundType extends bindings.MojoEnum {
  static const SystemSoundType click = const SystemSoundType._(0);

  const SystemSoundType._(int v) : super(v);

  static const Map<String, SystemSoundType> valuesMap = const {
    "click": click,
  };
  static const List<SystemSoundType> values = const [
    click,
  ];

  static SystemSoundType valueOf(String name) => valuesMap[name];

  factory SystemSoundType(int v) {
    switch (v) {
      case 0:
        return click;
      default:
        return null;
    }
  }

  static SystemSoundType decode(bindings.Decoder decoder0, int offset) {
    int v = decoder0.decodeUint32(offset);
    SystemSoundType result = new SystemSoundType(v);
    if (result == null) {
      throw new bindings.MojoCodecError(
          'Bad value $v for enum SystemSoundType.');
    }
    return result;
  }

  String toString() {
    switch(this) {
      case click:
        return 'SystemSoundType.click';
    }
  }

  int toJson() => mojoEnumValue;
}



class _SystemSoundPlayParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  SystemSoundType type = null;

  _SystemSoundPlayParams() : super(kVersions.last.size);

  static _SystemSoundPlayParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SystemSoundPlayParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SystemSoundPlayParams result = new _SystemSoundPlayParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
        result.type = SystemSoundType.decode(decoder0, 8);
        if (result.type == null) {
          throw new bindings.MojoCodecError(
            'Trying to decode null union for non-nullable SystemSoundType.');
        }
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeEnum(type, 8);
  }

  String toString() {
    return "_SystemSoundPlayParams("
           "type: $type" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["type"] = type;
    return map;
  }
}


class SystemSoundPlayResponseParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  bool success = false;

  SystemSoundPlayResponseParams() : super(kVersions.last.size);

  static SystemSoundPlayResponseParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static SystemSoundPlayResponseParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    SystemSoundPlayResponseParams result = new SystemSoundPlayResponseParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.success = decoder0.decodeBool(8, 0);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeBool(success, 8, 0);
  }

  String toString() {
    return "SystemSoundPlayResponseParams("
           "success: $success" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["success"] = success;
    return map;
  }
}

const int _SystemSound_playName = 0;

abstract class SystemSound {
  static const String serviceName = "flutter::platform::SystemSound";
  dynamic play(SystemSoundType type,[Function responseFactory = null]);
}


class _SystemSoundProxyImpl extends bindings.Proxy {
  _SystemSoundProxyImpl.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) : super.fromEndpoint(endpoint);

  _SystemSoundProxyImpl.fromHandle(core.MojoHandle handle) :
      super.fromHandle(handle);

  _SystemSoundProxyImpl.unbound() : super.unbound();

  static _SystemSoundProxyImpl newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For _SystemSoundProxyImpl"));
    return new _SystemSoundProxyImpl.fromEndpoint(endpoint);
  }

  void handleResponse(bindings.ServiceMessage message) {
    switch (message.header.type) {
      case _SystemSound_playName:
        var r = SystemSoundPlayResponseParams.deserialize(
            message.payload);
        if (!message.header.hasRequestId) {
          proxyError("Expected a message with a valid request Id.");
          return;
        }
        Completer c = completerMap[message.header.requestId];
        if (c == null) {
          proxyError(
              "Message had unknown request Id: ${message.header.requestId}");
          return;
        }
        completerMap.remove(message.header.requestId);
        if (c.isCompleted) {
          proxyError("Response completer already completed");
          return;
        }
        c.complete(r);
        break;
      default:
        proxyError("Unexpected message type: ${message.header.type}");
        close(immediate: true);
        break;
    }
  }

  String toString() {
    var superString = super.toString();
    return "_SystemSoundProxyImpl($superString)";
  }
}


class _SystemSoundProxyCalls implements SystemSound {
  _SystemSoundProxyImpl _proxyImpl;

  _SystemSoundProxyCalls(this._proxyImpl);
    dynamic play(SystemSoundType type,[Function responseFactory = null]) {
      var params = new _SystemSoundPlayParams();
      params.type = type;
      return _proxyImpl.sendMessageWithRequestId(
          params,
          _SystemSound_playName,
          -1,
          bindings.MessageHeader.kMessageExpectsResponse);
    }
}


class SystemSoundProxy implements bindings.ProxyBase {
  final bindings.Proxy impl;
  SystemSound ptr;

  SystemSoundProxy(_SystemSoundProxyImpl proxyImpl) :
      impl = proxyImpl,
      ptr = new _SystemSoundProxyCalls(proxyImpl);

  SystemSoundProxy.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) :
      impl = new _SystemSoundProxyImpl.fromEndpoint(endpoint) {
    ptr = new _SystemSoundProxyCalls(impl);
  }

  SystemSoundProxy.fromHandle(core.MojoHandle handle) :
      impl = new _SystemSoundProxyImpl.fromHandle(handle) {
    ptr = new _SystemSoundProxyCalls(impl);
  }

  SystemSoundProxy.unbound() :
      impl = new _SystemSoundProxyImpl.unbound() {
    ptr = new _SystemSoundProxyCalls(impl);
  }

  factory SystemSoundProxy.connectToService(
      bindings.ServiceConnector s, String url, [String serviceName]) {
    SystemSoundProxy p = new SystemSoundProxy.unbound();
    s.connectToService(url, p, serviceName);
    return p;
  }

  static SystemSoundProxy newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For SystemSoundProxy"));
    return new SystemSoundProxy.fromEndpoint(endpoint);
  }

  String get serviceName => SystemSound.serviceName;

  Future close({bool immediate: false}) => impl.close(immediate: immediate);

  Future responseOrError(Future f) => impl.responseOrError(f);

  Future get errorFuture => impl.errorFuture;

  int get version => impl.version;

  Future<int> queryVersion() => impl.queryVersion();

  void requireVersion(int requiredVersion) {
    impl.requireVersion(requiredVersion);
  }

  String toString() {
    return "SystemSoundProxy($impl)";
  }
}


class SystemSoundStub extends bindings.Stub {
  SystemSound _impl = null;

  SystemSoundStub.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint, [this._impl])
      : super.fromEndpoint(endpoint);

  SystemSoundStub.fromHandle(core.MojoHandle handle, [this._impl])
      : super.fromHandle(handle);

  SystemSoundStub.unbound() : super.unbound();

  static SystemSoundStub newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For SystemSoundStub"));
    return new SystemSoundStub.fromEndpoint(endpoint);
  }


  SystemSoundPlayResponseParams _SystemSoundPlayResponseParamsFactory(bool success) {
    var mojo_factory_result = new SystemSoundPlayResponseParams();
    mojo_factory_result.success = success;
    return mojo_factory_result;
  }

  dynamic handleMessage(bindings.ServiceMessage message) {
    if (bindings.ControlMessageHandler.isControlMessage(message)) {
      return bindings.ControlMessageHandler.handleMessage(this,
                                                          0,
                                                          message);
    }
    assert(_impl != null);
    switch (message.header.type) {
      case _SystemSound_playName:
        var params = _SystemSoundPlayParams.deserialize(
            message.payload);
        var response = _impl.play(params.type,_SystemSoundPlayResponseParamsFactory);
        if (response is Future) {
          return response.then((response) {
            if (response != null) {
              return buildResponseWithId(
                  response,
                  _SystemSound_playName,
                  message.header.requestId,
                  bindings.MessageHeader.kMessageIsResponse);
            }
          });
        } else if (response != null) {
          return buildResponseWithId(
              response,
              _SystemSound_playName,
              message.header.requestId,
              bindings.MessageHeader.kMessageIsResponse);
        }
        break;
      default:
        throw new bindings.MojoCodecError("Unexpected message name");
        break;
    }
    return null;
  }

  SystemSound get impl => _impl;
  set impl(SystemSound d) {
    assert(_impl == null);
    _impl = d;
  }

  String toString() {
    var superString = super.toString();
    return "SystemSoundStub($superString)";
  }

  int get version => 0;
}


